from typing import List

from fastapi import HTTPException, status

from .models import User

def can_access_resource(user: User, allowed_roles: List[str], resource: str):
    forbidden_exception = (
        HTTPException(
            status_code=status.HTTP_403_FORBIDDEN,
            detail="Access to resource is forbidden",
        )
    )

    if not user.resources or resource not in user.resources:
        raise forbidden_exception

    api_resource = user.resources[resource]

    if not 'roles' in api_resource:
        raise forbidden_exception
    
    user_roles = api_resource.get('roles', [])
    has_access = any(role in allowed_roles for role in user_roles)
    
    if not has_access:
        raise forbidden_exception
    
    return user_roles



from typing import List

from fastapi import Depends, HTTPException, status
from fastapi.security import OAuth2PasswordBearer

from .hyperbloq_jwt import HyperbloqJWT, HyperbloqJWTError


oauth2_scheme = OAuth2PasswordBearer(tokenUrl="token")

class VerifyAccessToken:
    def __init__(
      self,
      pub_key: str = None,
      aud: List = None,
      alg: List = ['RS256'],
      decode_user=True,
    ):
        self.jwt = HyperbloqJWT(pub_key, aud, alg)
        self.decode_user = decode_user

    def __call__(self, token: str = Depends(oauth2_scheme)):
        try:
            self.jwt.verify_access_token(token)
            if self.decode_user:
              return self.jwt.get_current_user()

            return self.jwt.get_decoded_token()

        except HyperbloqJWTError as e:
          raise HTTPException(
            status_code=status.HTTP_401_UNAUTHORIZED,
            detail=e.message,
          )

        except Exception as e:
          raise HTTPException(
            status_code=status.HTTP_401_UNAUTHORIZED,
            detail="Could not validate credentials",
          )



from typing import List

from fastapi import HTTPException, status


def can_access_resource_with_token(
  token: dict,
  allowed_roles: List[str],
  resource: str,
):
    forbidden_exception = (
        HTTPException(
            status_code=status.HTTP_403_FORBIDDEN,
            detail="Access to resource is forbidden",
        )
    )
    
    resources = token.get("resource_access", None)
    if not resources or resource not in resources:
        raise forbidden_exception

    api_resource = resources[resource]

    if not 'roles' in api_resource:
        raise forbidden_exception
    
    user_roles = api_resource.get('roles', [])
    has_access = any(role in allowed_roles for role in user_roles)

    if not has_access:
        raise forbidden_exception
    
    return user_roles



from bson import ObjectId
from fastapi.encoders import jsonable_encoder
from pydantic import BaseModel
from pydantic import BaseModel, EmailStr, Field

from .py_object_id_field import PyObjectId


class User(BaseModel):
    id: str = Field(alias='userId')
    username: str
    profileId: PyObjectId
    name: str
    given_name: str
    family_name: str
    email:  EmailStr
    locale: str
    resources: dict
    email_verified: bool

    class Config:
        allow_population_by_field_name = True
        arbitrary_types_allowed = True
        json_encoders = {ObjectId: str}
        use_enum_values = True,
        schema_extra = {
            'example': {
                'id': 'bf7fe257-d107-4f0d-9408-8b338ac318e7',
                'username': 'superusername', 
                'profileId': '62c999287f39adde61ed8fbd',
                'name': 'Jose Doe',
                'given_name': 'Jose',
                'family_name': 'Doe',
                'email': 'jose.doe@gmail.com',
                'locale': 'es',
                'resources': {
                  'custom-resource-name-1': {
                    'roles': ['citizen'],
                  },
                  'custom-resource-name-2': {
                    'roles': ['citizen'],
                  },
                  'account': {
                    'roles': [
                      'manage-account',
                      'manage-account-links',
                      'view-profile',
                    ],
                  },
                },
                'email_verified': True,
            }
        }
    
    def to_json(self):
        return jsonable_encoder(self, exclude_none=True)

    def to_bson(self):
        return self.dict(by_alias=True, exclude_none=True)



from typing import List
from jwt import decode, ExpiredSignatureError

from .models import User


class HyperbloqJWTError(Exception):
    """Exception raised for errors in the jwt decode.

    Attributes:
        message -- explanation of the error
    """

    def __init__(
      self,
      message="Missing params for decoding jwt 'pub_key' and 'aud' are required",
    ):
        self.message = message
        super().__init__(self.message)


class HyperbloqJWT:
    def __init__(self, pub_key: str = None, aud: List = None, alg: List = ["RS256"]):
        if not pub_key or not aud or not alg:
            raise HyperbloqJWTError

        self.secret = f"-----BEGIN PUBLIC KEY-----\r\n{pub_key}\r\n-----END PUBLIC KEY-----"
        self.audience = aud
        self.algorithms = alg
        self.decoded_token = {}

    def verify_access_token(self, token):
        try:
            payload = decode(
                token,
                self.secret,
                audience=self.audience,
                algorithms=self.algorithms,
                options={
                  "verify_signature": True,
                  "verify_aud": True,
                  "verify_exp": True,
                },
            )
            self.decoded_token = payload
            return payload

        except ExpiredSignatureError:
            raise HyperbloqJWTError(message="Could not verify token")
    
    def get_current_user(self):
        data = self.decoded_token
        if not data:
            raise HyperbloqJWTError(message="Could not decode token")

        return User(**{
          "username": data.get("preferred_username"),
          "userId": data.get("sub"),
          "profileId": data.get("profile"),
          "name": data.get("name"),
          "given_name": data.get("given_name"),
          "family_name": data.get("family_name"),
          "email": data.get("email"),
          "locale": data.get("locale"),
          "resources": data.get("resource_access"),
          "email_verified": data.get("email_verified")
        })
    
    def get_decoded_token(self):
        if not self.decoded_token:
            raise HyperbloqJWTError(message="Could not decode token")

        return self.decoded_token



__version__ = '0.1.9'

from .verify_access_token import VerifyAccessToken  # noqa: F401
from .can_access_resource import can_access_resource  # noqa: F401
from .can_access_resource_with_token import can_access_resource_with_token  # noqa: F401



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['hyperutils', 'hyperutils.models']

package_data = \
{'': ['*']}

install_requires = \
['PyJWT[crypto]>=2.4.0,<3.0.0',
 'fastapi',
 'pydantic[email]>=1.9.1,<2.0.0',
 'pymongo>=4.1.1,<5.0.0']

setup_kwargs = {
    'name': 'hyperutils',
    'version': '0.1.9',
    'description': 'hyperbloq-utils',
    'long_description': None,
    'author': 'Lionel Cuevas',
    'author_email': 'lionel.c@hyperbloq.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
